/*
  main.cpp
  OpenTTD 32bpp Tile Tool
  Made by Luca Spiller 2005
  For more info on OpenTTD see http://www.openttd.org/
  
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the
 Free Software Foundation, Inc., 
 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <dirent.h>
#include <SDL/SDL.h>
#include <dirent.h>
#include <math.h>
using namespace std;

#include "tiles.h"
#include "font.h"

SDLFont *font;

int _loaded_tile_images = 0;
Tile _tiles_images[MAX_TILE_IMAGES][ZOOM_LEVELS];

bool _display_fl = 0;

Tile *tiles[5][5], *tile_to_place = NULL;
int tile_height[5][5];

int zoom = 1;

int HalfTileSizes[ZOOM_LEVELS][2] = {
	{ 128, 64 },
	{ 64, 32 },
	{ 32, 16 },
};

int TileOffset[ZOOM_LEVELS][2] = {
	{ 16, 32 },
	{ 8, 16 },
	{ 4 , 8 },
};


int SelectedTileX, SelectedTileY;
int xoff = 270, yoff = 20;
unsigned long SelectedTileExpiry = 0;

SDL_Surface *screen, *tile_select, *filled_tile_select;

int main(int argc, char *argv[])
{
	int done = 0;
	Uint8* keys;
	unsigned long cTime;
	
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
	{
		printf("Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

	screen = SDL_SetVideoMode(800, 600, 32, SDL_HWSURFACE|SDL_DOUBLEBUF);
	if ( screen == NULL )
	{
		printf("Unable to set 800x600 video: %s\n", SDL_GetError());
		exit(1);
	}
	
	font = initFont("font");
	
	{// read the data/ directory and load images
		DIR *dir = opendir("data");
		struct dirent *dirent;
	
		if (!LoadTileSet("blank"))
			return 0;
		
		_loaded_tile_images++;
		
		if (!LoadTileSet("big_tile_select"))
			return 0;
		
		if (dir != NULL) {
			while ((dirent = readdir(dir)) != NULL) {
				if (dirent->d_name[0] != '.' && strcmp(dirent->d_name, "blank") && strcmp(dirent->d_name, "big_tile_select")) {
					if (_loaded_tile_images++ >= MAX_TILE_IMAGES) {
						printf("Too many tile images :(\n");
						return 0;
					}
					
					if (!LoadTileSet(dirent->d_name))
						return 0;
					printf("Loaded tileset `%s` (%d)\n", dirent->d_name, _loaded_tile_images);
				}
			}
		}
	}
	
	tile_select = LoadImage("tile_select.bmp", 255, 0, 0);
	filled_tile_select = LoadImage("filled_tile_select.bmp", 255, 0, 0);
	
	{ // set the tiles to the blank ones initially
		int x, y;
		
		for (x = 0; x < 5; x++) {
			for (y = 0; y < 5; y++) {
				tiles[x][y] = _tiles_images[0];
				tile_height[x][y] = 0;
			}
		}
	}
	
	while(done == 0)
	{
		SDL_Event event;
		int x, y, i;
		bool kSpaceDown, kQDown, kADown;
		 cTime = SDL_GetTicks();

		// check for any events
		while (SDL_PollEvent(&event))
		{
			if (event.type == SDL_QUIT)
				done = 1;	

			if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE)
				done = 1;
				
			if (event.type == SDL_MOUSEBUTTONDOWN && event.button.button == SDL_BUTTON_LEFT) {
				int x, y, x2, y2;
				SDL_GetMouseState(&x, &y);
				if (_display_fl) {
					y2 = (int) floor((y - 10) / 15) + 1;
					if (y2 > 1 && y2 <= _loaded_tile_images) {
						printf("Clicked on %s\n", _tiles_images[y2][0].name);	
						tile_to_place = _tiles_images[y2];
						_display_fl = !_display_fl;
					}
				} else {
					if (y > 10 && y < 210 && x > screen->w - 210 && x < screen->w - 10) {	
						x2 = (screen->w - x - 10) / 40;
						y2 = (y - 10) / 40;
						printf("Tile X: %d Tile Y: %d\n", x2, y2);
						if (tile_to_place == NULL) {
							SelectedTileX = x2;
							SelectedTileY = y2;
							SelectedTileExpiry = cTime + 10000;
						} else {
							tiles[x2][y2] = tile_to_place;
							tile_height[x2][y2] = 0;
							tile_to_place = NULL;
						}
					}
				}
			}
				
		}

		// check key states
		keys = SDL_GetKeyState(NULL);
		if (keys[SDLK_UP]) { yoff -= 5; }
		if (keys[SDLK_DOWN]) { yoff += 5; }
		if (keys[SDLK_LEFT]) { xoff -= 5; }
		if (keys[SDLK_RIGHT]) { xoff += 5; }

		if (keys[SDLK_PAGEUP]) { if (zoom < ZOOM_LEVELS - 1) zoom++;}
		if (keys[SDLK_PAGEDOWN]) { if (zoom > 0) zoom--; }
		
		// change tile height
		if (!kQDown && SelectedTileExpiry != 0 && keys[SDLK_q]) {
			tile_height[SelectedTileX][SelectedTileY]++;
			SelectedTileExpiry = cTime + 15000;
			kQDown = true;
		} else if (kQDown && !keys[SDLK_q])
			kQDown = false;
			
		if (!kADown && SelectedTileExpiry != 0 && keys[SDLK_a]) {
			tile_height[SelectedTileX][SelectedTileY]--;
			SelectedTileExpiry = cTime + 15000;
			kADown = true;
		} else if (kADown && !keys[SDLK_a])
			kADown = false;			
		
		// show tile select menu
		if (!kSpaceDown && keys[SDLK_SPACE]) {
			_display_fl = !_display_fl;
			kSpaceDown = true;
		} else if (kSpaceDown && !keys[SDLK_SPACE])
			kSpaceDown = false;
		
		
		// Drawing stuff
		SDL_FillRect(screen, NULL, 0x000000);

		for (x = 0; x < 5; x++) {
			y = 0;
			for (y = 0; y < 5; y++) {
				SDL_Rect dest;
				
				// Isometric view		
				// base tiles
				dest.x = xoff + x* HalfTileSizes[zoom][0] - (HalfTileSizes[zoom][0] * y); // tile_x
				dest.y = yoff + y* HalfTileSizes[zoom][1] + (HalfTileSizes[zoom][1] * x); // tile_y
				SDL_BlitSurface(_tiles_images[0][zoom].image, NULL,screen, &dest);
				
				// actual tiles
				dest.x = xoff + x* HalfTileSizes[zoom][0] - (HalfTileSizes[zoom][0] * y) - (tile_height[x][y] * TileOffset[zoom][0]) + tiles[x][y][zoom].offset_x; // tile_x
				dest.y = yoff + y* HalfTileSizes[zoom][1] + (HalfTileSizes[zoom][1] * x) - (tile_height[x][y] * TileOffset[zoom][1]) + tiles[x][y][zoom].offset_y; // tile_y
				SDL_BlitSurface(tiles[x][y][zoom].image, NULL,screen, &dest);
				
				// Top down view
				dest.x = screen->w - 50 - x * 40;
				dest.y = 10 + y * 40;
				SDL_BlitSurface((tile_to_place == NULL) ? tile_select : filled_tile_select, NULL,screen, &dest);
			}
		}

		if (tile_to_place != NULL)
			drawString(screen, font, screen->w - 210, 220, "Select a position");
		else if (SelectedTileExpiry != 0) {
			drawString(screen, font, screen->w - 210, 220, "Q    Increase Height");
			drawString(screen, font, screen->w - 210, 235, "A    Decrease Height");
		} else if (!_display_fl) {
			drawString(screen, font, screen->w - 210, 250, "Spacebar              Draw a tile");
			drawString(screen, font, screen->w - 210, 265, "Arrow Keys           Move");
			drawString(screen, font, screen->w - 210, 280, "Page Up / Down    Zoom");
		}
						
		// list of loaded tiles
		if (_display_fl) {
			char buffer[255];
			y = 10;
			drawString(screen, font, 10, y, "Select a tile");
			for (i = 2; i <= _loaded_tile_images; i++) {
				y += 15;
				drawString(screen, font, 10, y, _tiles_images[i][0].name);
			}
		}
		
		// selected tile
		if (SelectedTileExpiry != 0 && cTime < SelectedTileExpiry)
		{
			SDL_Rect dest;
			dest.x = xoff + SelectedTileX * HalfTileSizes[zoom][0] - (HalfTileSizes[zoom][0] * SelectedTileY);
			dest.y = yoff + SelectedTileY * HalfTileSizes[zoom][1] + (HalfTileSizes[zoom][1] * SelectedTileX);
			SDL_BlitSurface(_tiles_images[1][zoom].image, NULL, screen, &dest);
		} else if (SelectedTileExpiry != 0 && cTime > SelectedTileExpiry) {
			SelectedTileExpiry = 0;
		}
		
		SDL_Delay(50);
		SDL_Flip(screen);
	}
	
	return 0;
}
